#ifndef CORE_MATH_H
#define CORE_MATH_H

#include "utils/Utility.h"

namespace Core::Math {
    template<typename T>
    T interpolate(const T& a, const T& b, float f) {
        return a * (1.0f - f) + b * f;
    }

    constexpr int roundUpLog2(int i) {
        if(i <= 0) {
            return 0;
        }
        int c = 1;
        while(((i - 1) >> c) > 0) {
            c++;
        }
        return c;
    }

    template<typename T>
    constexpr const T& min(const T& t) {
        return t;
    }

    template<typename T, typename... Args>
    constexpr const T& min(const T& t, Args&&... args) {
        const T& o = min(Core::forward<Args>(args)...);
        return t < o ? t : o;
    }

    template<typename T>
    constexpr const T& max(const T& t) {
        return t;
    }

    template<typename T, typename... Args>
    constexpr const T& max(const T& t, Args&&... args) {
        const T& o = max(Core::forward<Args>(args)...);
        return o < t ? t : o;
    }

    template<typename T>
    constexpr const T& clamp(const T& t, const T& borderA, const T& borderB) {
        const T& low = min(borderA, borderB);
        const T& high = max(borderA, borderB);
        return max(low, min(high, t));
    }

    constexpr float PI = 3.14159265358979323846f;

    void sinCos(float a, float& s, float& c);

    constexpr float radianToDegree(float radians) {
        return radians * (180.0f / PI);
    }

    constexpr float degreeToRadian(float degrees) {
        return degrees * (PI / 180.0f);
    }

    float squareRoot(float f);
}

#endif